#ifndef DEBUG_H
#define DEBUG_H

#undef AIOLI_DEBUG
//#define AIOLI_DEBUG

#ifdef AIOLI_DEBUG

#define CONFIG_DEBUG_SPINLOCK 1

#define PRINT_FUNCTION_NAME printk(KERN_ALERT "%s\n", __PRETTY_FUNCTION__)
#define PRINT_FUNCTION_EXIT printk(KERN_ALERT "%s exited\n", __PRETTY_FUNCTION__)

#define debug(f, a...) printk(KERN_ALERT "aIOLi: %s(): " f "\n", __PRETTY_FUNCTION__, ## a)

#define FC_GREEN "\e[1;32m"
#define FC_YELLOW "\e[1;33m"
#define FC_RED "\e[1;31m"
#define FC_BLUE "\e[1;34m"
#define FC_NORMAL "\e[m"

//#define print_general(COLOR, f, a...) printk(KERN_ALERT COLOR  "aIOLi: " f  FC_NORMAL "\n" , ## a)
#define print_general(COLOR, f, a...) printk(KERN_ALERT COLOR "aIOLi: %s(): " f  FC_NORMAL "\n" , __PRETTY_FUNCTION__,  ## a)

#define print_green(f, a...) print_general(FC_GREEN, f, ## a)
#define print_yellow(f, a...) print_general(FC_YELLOW, f, ## a)
#define print_red(f, a...) print_general(FC_RED, f, ## a)
#define print_blue(f, a...) print_general(FC_BLUE, f, ## a)

#define aioli_panic(f, a...)	\
	panic("%s: " FC_RED f FC_NORMAL, __PRETTY_FUNCTION__, ## a)

#define VERIFY_REQUEST(req) 						\
	if (req->sanity != 123456) {					\
		aioli_panic("It isn't a request!!!");			\
	}

static inline void xlist_add(struct list_head *new, struct list_head *head)
{
	PRINT_FUNCTION_NAME;
	
	__list_add(new, head, head->next);
}

static inline void xlist_add_tail(struct list_head *new, struct list_head *head)
{
	PRINT_FUNCTION_NAME;
	
	__list_add(new, head->prev, head);
}

static inline void xlist_del(struct list_head *entry)
{
	PRINT_FUNCTION_NAME;
	
	__list_del(entry->prev, entry->next);
	entry->next = entry;
	entry->prev = entry;
}

static inline void xdown(struct semaphore *sem)
{
	//PRINT_FUNCTION_NAME;
	down(sem);
	//PRINT_FUNCTION_EXIT;
}

static inline void xup(struct semaphore *sem)
{
	//PRINT_FUNCTION_NAME;
	up(sem);
	//PRINT_FUNCTION_EXIT;
}

#else //#ifdef AIOLI_DEBUG

#define PRINT_FUNCTION_NAME (void)(0)
#define PRINT_FUNCTION_EXIT (void)(0)
#define aioli_panic(f, a...) (void)(0)
#define debug(f, a...) (void)(0)
#define print_green(f, a...) (void)(0)
#define print_yellow(f, a...) (void)(0)
#define print_red(f, a...) (void)(0)
#define print_blue(f, a...) (void)(0)
#define VERIFY_REQUEST(req) (void)(0)

#define xlist_add list_add
#define xlist_add_tail list_add_tail
#define xlist_del list_del
#define xup up
#define xdown down
		
#endif //#ifdef AIOLI_DEBUG

#endif // #ifndef DEBUG_H
