/*
 * File:	request_cache.h
 * Created:	25 July 2005
 * License:	GPL version 2
 * Authors:
 *		Przemyslaw Sowa <sowa.przemyslaw (at) gmail.com>
 * 		Adrien Lebre <adrien.lebre (at) imag.fr>
 *
 * Description:
 *		This file is a part of the aIOLi kernel module implementation.
 *		Contains data structure declarations and function prototypes
 *		used in aIOLi kernel module.
 *
 *		Further information is available at http://aioli.imag.fr
 *
 * Contributors:
 *		BULL Open Source Division
 *		INRIA France
 * 
 *		This program is distributed in the hope that it will be useful,
 * 		but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * Changes:
 */

#ifndef REQUEST_CACHE_H
#define REQUEST_CACHE_H

struct request_t;

void timeline_add_req(struct request_t *req);

/*
 * WARNING: these two functions delete request from list before
 * returning it.
 */
struct request_t *timeline_oldest_req(void);
struct request_t *timeline_newest_req(void);

void timeline_del_req(struct request_t *req);
int timeline_empty(void);

struct list_head *timeline_lock(void);
void timeline_unlock(void);

struct request_t *request_cache_alloc(void);
struct request_t * request_constructor(struct file *filp, char __user *buf,
				size_t len);
void request_cache_free(struct request_t *req);
int request_cache_init(void);
void request_cache_cleanup(void);

void hashtable_add_req(struct request_t *req);
void hashtable_del_req(struct request_t *req);
void __hashtable_del_req(struct request_t *req);

struct list_head *hashtable_lock(int index);
void hashtable_unlock(int index);
int hashtable_get_size(void);

int relatedlist_display(struct list_head *list); // ADRIEN DEBUG FUNCTION

#endif // #ifndef  REQUEST_CACHE_H
